import asyncio
import time

from .app import app
from ..testcase import RealServerAsyncTestCase
from ...grpc_requests import AsyncClient, Client


class PerformanceTestCase(RealServerAsyncTestCase):
    app = app
    service = 'helloworld.Greeter'
    method = 'SayHello'

    async def test_async_request(self):
        client = AsyncClient(self.default_endpoint)
        time_to_sleep = 0.05
        request_count = 50
        max_time = time_to_sleep * request_count

        async def request():
            await client.unary_unary(self.service, self.method, {'name': "sinsky"})
            await asyncio.sleep(time_to_sleep)

        start_time = time.time()

        await asyncio.gather(*(request() for _ in range(request_count)))
        end_time = time.time()

        runtime = end_time - start_time
        print(f"async request {request_count} time and sleep {time_to_sleep} seconds processed time : {runtime}")
        self.assertTrue(runtime < max_time)
        await client._channel.close()

    def test_sync_request(self):
        client = Client(self.default_endpoint)
        time_to_sleep = 0.05
        request_count = 50
        max_time = time_to_sleep * request_count

        def request():
            client.unary_unary(self.service, self.method, {'name': "sinsky"})
            time.sleep(time_to_sleep)

        start_time = time.time()
        for _ in range(request_count):
            request()
        end_time = time.time()

        runtime = end_time - start_time
        print(f"sync request {request_count} time and sleep {time_to_sleep} seconds processed time : {runtime}")
        self.assertTrue(runtime > max_time)
        client._channel.close()



from .app import app
from .helloworld_pb2 import _GREETER
from ..testcase import RealServerAsyncTestCase
from ...grpc_requests.aio import StubAsyncClient


class UnaryUnaryTestCase(RealServerAsyncTestCase):
    reset_descriptor_pool = False
    app = app
    service = 'helloworld.Greeter'
    method = 'SayHello'

    def setUp(self):
        super(UnaryUnaryTestCase, self).setUp()

    async def test_success_request(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        result = await client.unary_unary(self.service, self.method, {'name': "sinsky"})

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('message'), 'Hello sinsky!')
        await client._channel.close()

    async def test_not_support_service(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_service = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.unary_unary(not_support_service, self.method, {'name': "sinsky"})
        exception_msg = f"{context.exception}"
        print(exception_msg)
        self.assertTrue(f"server doesn't support {not_support_service}" in exception_msg)
        for service in ('helloworld.Greeter'):
            self.assertTrue(service in exception_msg)
        await client._channel.close()

    async def test_not_support_method(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_method = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.unary_unary(self.service, not_support_method, {'name': "sinsky"})
        exception_msg = f"{context.exception}"
        print(exception_msg)
        self.assertTrue(f"{self.service} doesn't support {not_support_method} method" in exception_msg)
        for method in ('HelloEveryone', 'SayHello', 'SayHelloGroup', 'SayHelloOneByOne'):
            self.assertTrue(method in exception_msg)
        await client._channel.close()


class UnaryStreamTestCase(RealServerAsyncTestCase):
    app = app
    service = 'helloworld.Greeter'
    method = 'SayHelloGroup'
    reset_descriptor_pool = False

    async def test_success_request(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        _results = await client.unary_stream(self.service, self.method, {'name': "sinsky"})

        results = [x async for x in _results]
        self.assertEqual(4, len(results))
        names = ['a', 'b', 'c', 'd']

        for result, name in zip(results, names):
            self.assertIsInstance(result, dict)
            self.assertTrue(name in result.get('message'))
        await client._channel.close()

    async def test_not_support_service(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_service = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.unary_stream(not_support_service, self.method, {'name': "sinsky"})
        exception_msg = f"{context.exception}"
        # print(exception_msg)
        self.assertTrue(f"server doesn't support {not_support_service}" in exception_msg)
        for service in ('helloworld.Greeter',):
            self.assertTrue(service in exception_msg)
        await client._channel.close()

    async def test_not_support_method(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_method = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.unary_stream(self.service, not_support_method, {'name': "sinsky"})
        exception_msg = f"{context.exception}"
        # print(exception_msg)
        self.assertTrue(f"{self.service} doesn't support {not_support_method} method" in exception_msg)
        for method in ('HelloEveryone', 'SayHello', 'SayHelloGroup', 'SayHelloOneByOne'):
            self.assertTrue(method in exception_msg)
        await client._channel.close()


class StreamStreamTestCase(RealServerAsyncTestCase):
    app = app
    service = 'helloworld.Greeter'
    method = 'SayHelloOneByOne'
    reset_descriptor_pool = False

    async def test_success_request(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        names = ['sinsky', 'summer', 'wone']
        _results = await client.stream_stream(self.service, self.method, ({"name": name} for name in names))

        results = [x async for x in _results]
        self.assertEqual(len(names), len(results))

        for result, name in zip(results, names):
            self.assertIsInstance(result, dict)
            self.assertTrue(name in result.get('message'))
        await client._channel.close()

    async def test_not_support_service(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_service = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.stream_stream(not_support_service, self.method, [{'name': "sinsky"}])
        exception_msg = f"{context.exception}"

        self.assertTrue(f"server doesn't support {not_support_service}" in exception_msg)
        for service in ('helloworld.Greeter',):
            self.assertTrue(service in exception_msg)
        await client._channel.close()

    async def test_not_support_method(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_method = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.stream_stream(self.service, not_support_method, [{'name': "sinsky"}])
        exception_msg = f"{context.exception}"

        self.assertTrue(f"{self.service} doesn't support {not_support_method} method" in exception_msg)
        for method in ('HelloEveryone', 'SayHello', 'SayHelloGroup', 'SayHelloOneByOne'):
            self.assertTrue(method in exception_msg)

        await client._channel.close()


class StreamUnaryTestCase(RealServerAsyncTestCase):
    app = app
    service = 'helloworld.Greeter'
    method = 'HelloEveryone'
    reset_descriptor_pool = False

    async def test_success_request(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        names = ['sinsky', 'summer', 'wone']
        result = await client.stream_unary(self.service, self.method, ({"name": name} for name in names))
        self.assertIsInstance(result, dict)
        self.assertTrue(f"{names}" in result.get('message', ''))
        await client._channel.close()

    async def test_not_support_service(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_service = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.stream_unary(not_support_service, self.method, [{'name': "sinsky"}])
        exception_msg = f"{context.exception}"

        self.assertTrue(f"server doesn't support {not_support_service}" in exception_msg)
        for service in ('helloworld.Greeter',):
            self.assertTrue(service in exception_msg)
        await client._channel.close()

    async def test_not_support_method(self):
        client = StubAsyncClient(self.default_endpoint, [_GREETER])
        not_support_method = 'notSupport'
        with self.assertRaises(ValueError) as context:
            await client.stream_unary(self.service, not_support_method, [{'name': "sinsky"}])
        exception_msg = f"{context.exception}"
        print(exception_msg)
        self.assertTrue(f"{self.service} doesn't support {not_support_method} method" in exception_msg)
        for method in ('HelloEveryone', 'SayHello', 'SayHelloGroup', 'SayHelloOneByOne'):
            self.assertTrue(method in exception_msg)
        await client._channel.close()



from .aio import AsyncClient, ReflectionAsyncClient, StubAsyncClient, get_by_endpoint as async_get_by_endpoint
from .client import Client, ReflectionClient, StubClient, get_by_endpoint

__version__ = "0.1.10"



from .app import app
from ..testcase import RealServerAsyncTestCase
from ...grpc_requests.aio import AsyncClient


class CertificateAsyncTestCase(RealServerAsyncTestCase):
    app = app
    service = 'helloworld.Greeter'
    method = 'SayHello'
    tls = True

    async def test_success_request(self):
        client = AsyncClient(self.default_endpoint, ssl=True, credentials=dict(
            root_certificates=self.tls_config['certificate'],
        ))
        result = await client.unary_unary(self.service, self.method, {'name': "sinsky"})

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('message'), 'Hello sinsky!')
        await client._channel.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: helloworld.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10helloworld.proto\x12\nhelloworld\"\x1c\n\x0cHelloRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x1d\n\nHelloReply\x12\x0f\n\x07message\x18\x01 \x01(\t2\xa3\x02\n\x07Greeter\x12>\n\x08SayHello\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00\x12\x45\n\rSayHelloGroup\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00\x30\x01\x12\x45\n\rHelloEveryone\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00(\x01\x12J\n\x10SayHelloOneByOne\x12\x18.helloworld.HelloRequest\x1a\x16.helloworld.HelloReply\"\x00(\x01\x30\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'helloworld_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HELLOREQUEST._serialized_start=32
  _HELLOREQUEST._serialized_end=60
  _HELLOREPLY._serialized_start=62
  _HELLOREPLY._serialized_end=91
  _GREETER._serialized_start=94
  _GREETER._serialized_end=385
# @@protoc_insertion_point(module_scope)



from concurrent import futures
from grpc_reflection.v1alpha import reflection

import grpc
import logging
from ..test_protos.helloworld_pb2_grpc import GreeterServicer, add_GreeterServicer_to_server
from ..test_protos.helloworld_pb2 import HelloReply, DESCRIPTOR

class Greeter(GreeterServicer):

    def SayHello(self, request, context):
        """
        Unary-Unary
        Sends a HelloReply based on a HelloRequest.
        """
        return HelloReply(message=f"Hello, {request.name}!")

    def SayHelloGroup(self, request, context):
        """
        Unary-Stream
        Streams a series of HelloReplies based on the names in a HelloRequest.
        """
        names = request.name
        for name in names.split():
            yield HelloReply(message=f"Hello, {name}!")

    def HelloEveryone(self, request_iterator, context):
        """
        Stream-Unary
        Sends a HelloReply based on the name recieved from a stream of
        HelloRequests.
        """
        names = []
        for request in request_iterator:
            names.append(request.name)
        names_string = " ".join(names)
        return HelloReply(message=f"Hello, {names_string}!")

    def SayHelloOneByOne(self, request_iterator, context):
        """
        Stream-Stream
        Streams HelloReplies in response to a stream of HelloRequests.
        """
        for request in request_iterator:
            yield HelloReply(message=f"Hello {request.name}")


class HelloWorldServer():

    server = None

    def __init__(self, port: str):
        self.server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        add_GreeterServicer_to_server(Greeter(), self.server)
        SERVICE_NAMES = (
            DESCRIPTOR.services_by_name['Greeter'].full_name,
            reflection.SERVICE_NAME,
        )
        reflection.enable_server_reflection(SERVICE_NAMES, self.server)
        self.server.add_insecure_port(f'[::]:{port}')

    def serve(self):
        logging.warning('Start the server?')
        self.server.start()
        logging.warning('Server running')
        self.server.wait_for_termination()

    def shutdown(self):
        self.server.stop(grace=3)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.shutdown()


if __name__ == "__main__":
    server = HelloWorldServer("50051")
    server.serve()



import re

from setuptools import setup

with open("src/grpc_requests/__init__.py", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

with open("requirements.txt", encoding="utf8") as f:
    requirements = [line for line in f.readlines()]

setup(
    name='grpc_requests',
    version=version,
    install_requires=requirements,
)
